/*
 *  linux/include/linux/cooperative.h
 *
 *  Copyright (C) 2004 Dan Aloni
 *
 *  This header gathers the functions and variables in Cooperative Mode 
 *  when CONFIG_COOPERATIVE is defined.
 */
#ifndef __LINUX_COOPERATIVE_LINUX_H__
#define __LINUX_COOPERATIVE_LINUX_H__

#include <linux/config.h>
#include <linux/cooperative.h>
#include <linux/list.h>

#ifdef CONFIG_COOPERATIVE

typedef struct {
	struct list_head node;
	co_message_t msg;
} co_message_node_t;

extern void co_debug(const char *fmt, ...);
extern void co_printk(const char *line);

extern void co_callback(unsigned long flags);
extern void co_switch_wrapper(void);
extern void co_idle_processor(void);
extern void co_terminate(co_termination_reason_t reason);
extern void co_free_pages(unsigned long vaddr, int order);
extern int co_alloc_pages(unsigned long vaddr, int order);
extern void co_start_kernel(void);
extern void co_arch_start_kernel(void);
extern void co_handle_jiffies(long count);

extern void co_send_message(co_module_t from, 
			    co_module_t to,
			    co_priority_t priority,
			    co_message_type_t type,
			    unsigned long size,
			    const char *data);
extern unsigned long co_get_host_time(void);
extern co_message_t *co_send_message_save(unsigned long *flags);
extern co_message_t *co_get_message_save(unsigned long *flags);
extern void co_send_message_restore(unsigned long flags);

extern void cocd_interrupt(void);

extern void co_handle_incoming_messages(void);
extern void co_handle_incoming_message(co_message_node_t *message);
extern void co_queue_incoming_message(co_message_node_t *message);
extern int co_get_message(co_message_node_t **message, co_device_t device);
extern void co_free_message(co_message_node_t *message);

extern int co_passage_page_held(void);
extern void co_passage_page_acquire(unsigned long *flags);
extern void co_passage_page_release(unsigned long flags);

#define co_passage_page_assert_valid() do {	\
	if (co_passage_page_held())		\
            BUG();				\
} while (0);

extern char co_boot_parameters[CO_BOOTPARAM_STRING_LENGTH];
extern unsigned long co_core_end;
extern unsigned long co_memory_size;
extern void *co_initrd;
extern unsigned long co_initrd_size;

#define cooperative_mode_enabled()     1

#else

#define co_printk(line)                do {} while (0)
#define co_idle_processor              (void*)NULL
#define co_terminate(reason)           do {} while (0)
#define co_alloc_pages(vaddr,order)    0
#define co_get_host_time()             0
#define co_boot_parameters             (char*)NULL
#define co_memory_size                 PAGE_SIZE
#define cooperative_mode_enabled()     0
#define proxy_interrupt                ((char*)NULL)

#endif

#endif
